#include "Datenbank.h"
#include <iostream>
#include <Zeit.h>

// Inhalt der LSDatenbank Klasse aus Datenbank.h
// Konstruktor
KSDatenbank::KSDatenbank( InitDatei *zIni )
    : ReferenceCounter()
{
    datenbank = new Datenbank( zIni->zWert( "DBBenutzer" )->getText(), zIni->zWert( "DBPasswort" )->getText(),
                               zIni->zWert( "DBName" )->getText(), zIni->zWert( "DBIP" )->getText(),
                               (unsigned short)TextZuInt( zIni->zWert( "DBPort" )->getText(), 10 ) );
    if( !datenbank->istOk() )
    {
        std::cout << "KS: Die Verbindung zur Datenbank konnte nicht hergestellt werden.\nDas Programm wird beendet.";
        exit( 1 );
    }
    InitializeCriticalSection( &cs );
    Text befehl = "SELECT port, admin_port  FROM server WHERE id = ";
    befehl += zIni->zWert( "ServerId" )->getText();
    lock();
    datenbank->befehl( befehl );
    Result res = datenbank->getResult();
    unlock();
    if( res.zeilenAnzahl == 1 )
    {
        zIni->addWert( "ServerPort", res.values[ 0 ] );
        zIni->addWert( "AdminServerPort", res.values[ 1 ] );
    }
    res.destroy();
}

// Destruktor
KSDatenbank::~KSDatenbank()
{
    datenbank->release();
    DeleteCriticalSection( &cs );
}

// nicht constant
void KSDatenbank::lock()
{
    EnterCriticalSection( &cs );
}

void KSDatenbank::unlock()
{
    LeaveCriticalSection( &cs );
}

int KSDatenbank::istAdministrator( const char *name, const char *passwort )
{
    Text *befehl = new Text( "SELECT id FROM benutzer WHERE name = '" );
    Text n( name );
    n.ersetzen( "'", "''" );
    befehl->append( (char *)n );
    befehl->append( "' AND passwort = md5( '" );
    Text p( passwort );
    p.ersetzen( "'", "''" );
    befehl->append( (char *)p );
    befehl->append( "' )" );
    lock();
    datenbank->befehl( befehl->getText() );
    Result res = datenbank->getResult();
    unlock();
    befehl->release();
    int ret = 0;
    if( res.zeilenAnzahl > 0 )
        ret = TextZuInt( res.values[ 0 ].getText(), 10 );
    res.destroy();
    return ret;
}

bool KSDatenbank::adminHatRecht( int id, int recht )
{
    Text *befehl = new Text( "SELECT * FROM benutzer_rechte WHERE benutzer_id = " );
    befehl->append( id );
    befehl->append( " AND rechte_id = " );
    befehl->append( recht );
    lock();
    datenbank->befehl( befehl->getText() );
    int ret = datenbank->getZeilenAnzahl();
    unlock();
    befehl->release();
    return ret != 0;
}

bool KSDatenbank::proveKlient( int num, int sNum )
{
    Text *befehl = new Text( "SELECT * FROM server_client WHERE server_id = " );
    befehl->append( sNum );
    befehl->append( " AND client_id = " );
    befehl->append( num );
    lock();
    datenbank->befehl( befehl->getText() );
    int ret = datenbank->getZeilenAnzahl();
    unlock();
    befehl->release();
    return ret == 1;
}

Text *KSDatenbank::getKlientKey( int cId )
{
    lock();
    if( !datenbank->befehl( Text( "SELECT schluessel FROM client WHERE id = " ) += cId ) )
    {
        unlock();
        return 0;
    }
    Result res = datenbank->getResult();
    unlock();
    if( !res.zeilenAnzahl )
    {
        res.destroy();
        return 0;
    }
    Text *ret = new Text( res.values[ 0 ].getText() );
    res.destroy();
    return ret;
}

void KSDatenbank::unregisterKlient( int num, int sNum )
{
    Text *befehl = new Text( "DELETE FROM server_client WHERE client_id = " );
    befehl->append( num );
    befehl->append( " AND server_id = " );
    befehl->append( sNum );
    lock();
    datenbank->befehl( befehl->getText() );
    unlock();
    befehl->release();
}

bool KSDatenbank::setServerStatus( int id, int status )
{
    Text *befehl = new Text( "UPDATE server SET server_status_id = " );
    *befehl += status;
    *befehl += "WHERE id = ";
    *befehl += id;
    lock();
    if( !datenbank->befehl( befehl->getText() ) )
    {
        unlock();
        befehl->release();
        return 0;
    }
    bool ret = datenbank->getZeilenAnzahl() != 0;
    unlock();
    befehl->release();
    return ret;
}

bool KSDatenbank::setMaxKarten( int id, int maxK )
{
    Text *befehl = new Text( "UPDATE server SET max_tasks = " );
    befehl->append( maxK );
    befehl->append( " WHERE id = " );
    befehl->append( id );
    lock();
    if( !datenbank->befehl( befehl->getText() ) )
    {
        unlock();
        befehl->release();
        return 0;
    }
    bool ret = datenbank->getZeilenAnzahl() > 0;
    unlock();
    befehl->release();
    return ret;
}

bool KSDatenbank::serverIstNichtPausiert( int id )
{
    Text *befehl = new Text( "SELECT server_status_id FROM server WHERE id = " );
    befehl->append( id );
    lock();
    if( !datenbank->befehl( befehl->getText() ) )
    {
        unlock();
        befehl->release();
        return 0;
    }
    Result res = datenbank->getResult();
    unlock();
    befehl->release();
    if( !res.zeilenAnzahl )
    {
        res.destroy();
        return 0;
    }
    bool ret = (int)res.values[ 0 ] == 3;
    res.destroy();
    return ret;
}

bool KSDatenbank::proveKarte( int id, int sNum )
{
    Text befehl = "SELECT id FROM karte WHERE id = ";
    befehl += id;
    befehl += " AND server_karten_id = ";
    befehl += sNum;
    lock();
    if( !datenbank->befehl( befehl ) )
    {
        unlock();
        return 0;
    }
    bool ret = datenbank->getZeilenAnzahl() != 0;
    unlock();
    return ret;
}

int KSDatenbank::getUpdateKarte( int serverId )
{
    Text befehl = "SELECT id FROM karte WHERE erlaubt = FALSE AND server_karten_id = ";
    befehl += serverId;
    lock();
    if( !datenbank->befehl( befehl ) )
    {
        unlock();
        return 0;
    }
    Result r = datenbank->getResult();
    unlock();
    for( int i = 0; i < r.zeilenAnzahl; i++ )
    {
        int karte = r.values[ i ];
        befehl = "SELECT karte_update_warten( ";
        befehl += karte;
        befehl += " )";
        lock();
        if( !datenbank->befehl( befehl ) )
        {
            unlock();
            continue;
        }
        Result res = datenbank->getResult();
        unlock();
        if( !res.zeilenAnzahl )
        {
            res.destroy();
            continue;
        }
        if( res.values[ 0 ].istGleich( "f" ) )
        {
            res.destroy();
            r.destroy();
            return karte;
        }
        res.destroy();
    }
    r.destroy();
    return 0;
}

bool KSDatenbank::endUpdateKarte( int karteId )
{
    Text befehl = "SELECT karte_update_ende( ";
    befehl += karteId;
    befehl += " )";
    lock();
    if( !datenbank->befehl( befehl ) )
    {
        unlock();
        return 0;
    }
    Result r = datenbank->getResult();
    unlock();
    if( !r.zeilenAnzahl )
    {
        r.destroy();
        return 0;
    }
    bool ret = r.values[ 0 ].istGleich( "t" );
    r.destroy();
    return ret;
}

bool KSDatenbank::getEditorServerPortIp( int karteId, int *port, Text *ip )
{
    if( !port || !ip )
        return 0;
    Text befehl = "SELECT a.port, a.ip FROM server a, karte b WHERE a.server_typ_name = 'editor' AND a.id = b.server_editor_id AND b.id = ";
    befehl += karteId;
    lock();
    if( !datenbank->befehl( befehl ) )
    {
        unlock();
        return 0;
    }
    Result r = datenbank->getResult();
    unlock();
    if( !r.zeilenAnzahl )
    {
        r.destroy();
        return 0;
    }
    *port = r.values[ 0 ];
    ip->setText( r.values[ 1 ] );
    r.destroy();
    return 1;
}

bool KSDatenbank::setKarteSpielerAnzahl( int karteId, int sAnz )
{
    Text befehl = "UPDATE karte SET max_spieler = ";
    befehl += sAnz;
    befehl += " WHERE id = ";
    befehl += karteId;
    lock();
    if( !datenbank->befehl( befehl ) )
    {
        unlock();
        return 0;
    }
    int anz = datenbank->getZeilenAnzahl();
    unlock();
    if( !anz )
        return 0;
    return 1;
}

// constant
Text *KSDatenbank::getLetzterFehler() const
{
    return datenbank->getLetzterFehler();
}